@@ -42,7 +42,8 @@ let rec expr_of_typ ?decl typ =
       [%expr Ppx_deriving_runtime.Array.map [%e expr_of_typ ?decl typ]]
     | true, [%type: [%t? typ] option] ->
       [%expr function None -> None | Some x -> Some ([%e expr_of_typ ?decl typ] x)]
-    | true, [%type: ([%t? ok_t], [%t? err_t]) Result.result] ->
+    | true, ([%type: ([%t? ok_t], [%t? err_t]) result] |
+             [%type: ([%t? ok_t], [%t? err_t]) Result.result]) ->
       [%expr
         function
         | Result.Ok ok -> Result.Ok ([%e expr_of_typ ?decl ok_t] ok)
@@ -58,12 +59,26 @@ let rec expr_of_typ ?decl typ =
   | { ptyp_desc = Ptyp_variant (fields, _, _); ptyp_loc } ->
     let cases =
       fields |> List.map (fun field ->
+        let pat_variant label popt =
+#if OCAML_VERSION < (4, 06, 0)
+          Pat.variant label popt
+#else
+          Pat.variant label.txt popt
+#endif
+        in
+        let exp_variant label popt =
+#if OCAML_VERSION < (4, 06, 0)
+          Exp.variant label popt
+#else
+          Exp.variant label.txt popt
+#endif
+        in
         match field with
         | Rtag (label, _, true (*empty*), []) ->
-          Exp.case (Pat.variant label None) (Exp.variant label None)
+          Exp.case (pat_variant label None) (exp_variant label None)
         | Rtag (label, _, false, [typ]) ->
-          Exp.case (Pat.variant label (Some [%pat? x]))
-                   (Exp.variant label (Some [%expr [%e expr_of_typ ?decl typ] x]))
+          Exp.case (pat_variant label (Some [%pat? x]))
+                   (exp_variant label (Some [%expr [%e expr_of_typ ?decl typ] x]))
         | Rinherit ({ ptyp_desc = Ptyp_constr (tname, _) } as typ) -> begin
           match decl with
           | None -> 
@@ -119,8 +134,9 @@ let str_of_type ~options ~path ({ ptype_loc = loc } as type_decl) =
       raise_errorf ~loc "%s cannot be derived for open types" deriver
   in
   let polymorphize = Ppx_deriving.poly_fun_of_type_decl type_decl in
-  [Vb.mk (pvar (Ppx_deriving.mangle_type_decl (`Prefix deriver) type_decl))
-               (polymorphize mapper)]
+  [Vb.mk ~attrs:[Ppx_deriving.attr_warning [%expr "-39"]]
+         (pvar (Ppx_deriving.mangle_type_decl (`Prefix deriver) type_decl))
+         (polymorphize mapper)]
 
 let sig_of_type ~options ~path type_decl =
   parse_options options;
