@@ -12,10 +12,25 @@ open Ast_convenience
 let deriver = "show"
 let raise_errorf = Ppx_deriving.raise_errorf
 
+type options = { with_path : bool }
+
+(* The option [with_path] controls whether a full path should be displayed
+   as part of data constructor names and record field names. (In the case
+   of record fields, it is displayed only as part of the name of the first
+   field.) By default, this option is [true], which means that full paths
+   are shown. *)
+
+let expand_path show_opts ~path name =
+  let path = if show_opts.with_path then path else [] in
+  Ppx_deriving.expand_path ~path name
+
 let parse_options options =
+  let with_path = ref true in
   options |> List.iter (fun (name, expr) ->
     match name with
-    | _ -> raise_errorf ~loc:expr.pexp_loc "%s does not support option %s" deriver name)
+    | "with_path" -> with_path := Ppx_deriving.Arg.(get_expr ~deriver bool) expr
+    | _ -> raise_errorf ~loc:expr.pexp_loc "%s does not support option %s" deriver name);
+  { with_path = !with_path }
 
 let attr_nobuiltin attrs =
   Ppx_deriving.(attrs |> attr ~deriver "nobuiltin" |> Arg.get_flag ~deriver)
@@ -42,7 +57,7 @@ let wrap_printer quoter printer =
     [%expr (let fprintf = Format.fprintf in [%e printer]) [@ocaml.warning "-26"]]
 
 let pp_type_of_decl ~options ~path type_decl =
-  parse_options options;
+  let _ = parse_options options in
   let typ = Ppx_deriving.core_type_of_type_decl type_decl in
   Ppx_deriving.poly_arrow_of_type_decl
     (fun var -> [%type: Format.formatter -> [%t var] -> Ppx_deriving_runtime.unit])
@@ -50,7 +65,7 @@ let pp_type_of_decl ~options ~path type_decl =
     [%type: Format.formatter -> [%t typ] -> Ppx_deriving_runtime.unit]
 
 let show_type_of_decl ~options ~path type_decl =
-  parse_options options;
+  let _ = parse_options options in
   let typ = Ppx_deriving.core_type_of_type_decl type_decl in
   Ppx_deriving.poly_arrow_of_type_decl
     (fun var -> [%type: Format.formatter -> [%t var] -> Ppx_deriving_runtime.unit])
@@ -58,7 +73,7 @@ let show_type_of_decl ~options ~path type_decl =
     [%type: [%t typ] -> Ppx_deriving_runtime.string]
 
 let sig_of_type ~options ~path type_decl =
-  parse_options options;
+  let _ = parse_options options in
   [Sig.value (Val.mk (mknoloc (Ppx_deriving.mangle_type_decl (`Prefix "pp") type_decl))
               (pp_type_of_decl ~options ~path type_decl));
    Sig.value (Val.mk (mknoloc (Ppx_deriving.mangle_type_decl (`Prefix "show") type_decl))
@@ -120,7 +135,8 @@ let rec expr_of_typ quoter typ =
             Format.pp_print_string fmt "(Some ";
             [%e expr_of_typ typ] x;
             Format.pp_print_string fmt ")"]
-      | true, [%type: ([%t? ok_t],[%t? err_t]) Result.result] ->
+      | true, ([%type: ([%t? ok_t], [%t? err_t]) result] |
+               [%type: ([%t? ok_t], [%t? err_t]) Result.result]) ->
         [%expr
           function
           | Result.Ok ok ->
@@ -160,9 +176,15 @@ let rec expr_of_typ quoter typ =
         fields |> List.map (fun field ->
           match field with
           | Rtag (label, _, true (*empty*), []) ->
+#if OCAML_VERSION >= (4, 06, 0)
+            let label = label.txt in
+#endif
             Exp.case (Pat.variant label None)
                      [%expr Format.pp_print_string fmt [%e str ("`" ^ label)]]
           | Rtag (label, _, false, [typ]) ->
+#if OCAML_VERSION >= (4, 06, 0)
+            let label = label.txt in
+#endif
             Exp.case (Pat.variant label (Some [%pat? x]))
                      [%expr Format.fprintf fmt [%e str ("`" ^ label ^ " (@[<hov>")];
                             [%e expr_of_typ typ] x;
@@ -182,7 +204,7 @@ let rec expr_of_typ quoter typ =
                    deriver (Ppx_deriving.string_of_core_type typ)
 
 let str_of_type ~options ~path ({ ptype_loc = loc } as type_decl) =
-  parse_options options;
+  let show_opts = parse_options options in
   let quoter = Ppx_deriving.create_quoter () in
   let path = Ppx_deriving.path_of_type_decl ~path type_decl in
   let prettyprinter =
@@ -192,7 +214,10 @@ let str_of_type ~options ~path ({ ptype_loc = loc } as type_decl) =
     | Ptype_variant constrs, _ ->
       let cases =
         constrs |> List.map (fun { pcd_name = { txt = name' }; pcd_args; pcd_attributes } ->
-          let constr_name = Ppx_deriving.expand_path ~path name' in
+          let constr_name =
+            expand_path show_opts ~path name'
+          in
+
           match attr_printer pcd_attributes, pcd_args with
           | Some printer, Pcstr_tuple(args) ->
             let rec range from_idx to_idx =
@@ -260,7 +285,7 @@ let str_of_type ~options ~path ({ ptype_loc = loc } as type_decl) =
     | Ptype_record labels, _ ->
       let fields =
         labels |> List.mapi (fun i { pld_name = { txt = name }; pld_type; pld_attributes } ->
-          let field_name = if i = 0 then Ppx_deriving.expand_path ~path name else name in
+          let field_name = if i = 0 then expand_path show_opts ~path name else name in
           let pld_type = {pld_type with ptyp_attributes=pld_attributes@pld_type.ptyp_attributes} in
           [%expr
             Format.fprintf fmt "@[%s =@ " [%e str field_name];
