@@ -153,15 +153,29 @@ let test_mrec ctxt =
   let e1 =  B { x = 12; r = F 16 } in
   assert_equal ~printer "(Test_deriving_show.B\n   { Test_deriving_show.x = 12; r = (Test_deriving_show.F 16) })" (show_foo e1)
 
-type i_has_result = I_has of (bool, string) Result.result [@@deriving show]
+
+#if OCAML_VERSION >= (4, 03, 0)
+type i_has_result = I_has of (bool, string) result [@@deriving show]
 
 let test_result ctxt =
   assert_equal ~printer "(Ok 100)"
-    ([%show: (int, bool) Result.result] (Result.Ok 100));
+    ([%show: (int, bool) result] (Ok 100));
   assert_equal ~printer "(Test_deriving_show.I_has (Ok true))"
-    (show_i_has_result (I_has (Result.Ok true)));
+    (show_i_has_result (I_has (Ok true)));
   assert_equal ~printer "(Test_deriving_show.I_has (Error \"err\"))"
-    (show_i_has_result (I_has (Result.Error "err")))
+    (show_i_has_result (I_has (Error "err")))
+#endif
+
+type i_has_result_result = I_has of (bool, string) Result.result [@@deriving show]
+
+let test_result_result ctxt =
+  let open Result in
+  assert_equal ~printer "(Ok 100)"
+    ([%show: (int, bool) result] (Result.Ok 100));
+  assert_equal ~printer "(Test_deriving_show.I_has (Ok true))"
+    (show_i_has_result_result (I_has (Ok true)));
+  assert_equal ~printer "(Test_deriving_show.I_has (Error \"err\"))"
+    (show_i_has_result_result (I_has (Error "err")))
 
 type es =
   | ESBool of (bool [@nobuiltin])
@@ -214,6 +228,17 @@ let test_variant_printer ctxt =
   assert_equal ~printer
     "fourth: 8 4" (show_variant_printer (Fourth(8,4)))
 
+type no_full    = NoFull   of int [@@deriving show { with_path = false }]
+type with_full  = WithFull of int [@@deriving show { with_path = true  }]
+module WithFull = struct
+  type t = A of int [@@deriving show ]
+end
+let test_paths_printer ctxt =
+  assert_equal ~printer "(NoFull 1)"   (show_no_full   (NoFull 1));
+  assert_equal ~printer "(Test_deriving_show.WithFull 1)" (show_with_full (WithFull 1));
+  assert_equal ~printer "(Test_deriving_show.WithFull.A 1)" (WithFull.show (WithFull.A 1));
+  ()
+
 let suite = "Test deriving(show)" >::: [
     "test_alias"           >:: test_alias;
     "test_variant"         >:: test_variant;
@@ -232,5 +257,9 @@ let suite = "Test deriving(show)" >::: [
     "test_std_shadowing"   >:: test_std_shadowing;
     "test_poly_app"        >:: test_poly_app;
     "test_variant_printer" >:: test_variant_printer;
-    "test_result"          >:: test_result
+    "test_paths"           >:: test_paths_printer;
+#if OCAML_VERSION >= (4, 03, 0)
+    "test_result"          >:: test_result;
+#endif
+    "test_result_result"   >:: test_result_result;
   ]
